package org.expeditee.items.widgets;

import java.awt.Component;
import java.awt.Container;
import java.awt.Graphics2D;
import java.awt.Point;

import javax.swing.JComponent;

import org.expeditee.gio.EcosystemManager.Ecosystem;
import org.expeditee.gio.swing.SwingMiscManager;
import org.expeditee.items.Text;

public abstract class SwingWidget extends Widget
{
	// A flag for signifying whether the swing components are ready to paint.
	// If the swing components has not been layed out, if they are painted they
	// will not draw in the correct positions.
	// Also for setting AWT and Swing -Related drawing options after
	// construction
	private boolean _isReadyToPaint = false;

	protected JComponent _swingComponent;
	
	protected SwingWidget(Text source, JComponent component, int minWidth, int maxWidth, int minHeight, int maxHeight)
	{
		super(source, minWidth, maxWidth, minHeight, maxHeight);

		if (component == null) throw new NullPointerException("component");

		_swingComponent = component;
		onBoundsChanged();
	}

	public JComponent getComponent()
	{
		return _swingComponent;
	}
	
	@Override
	protected void onMoved()
	{
		// Do nothing
	}
	
	@Override
	protected void onSizeChanged()
	{
    	onBoundsChanged();
    	layout(_swingComponent);
	}

	public final void onBoundsChanged()
	{
		if (_swingComponent == null) return;
		
		if (isFixedSize()) {
			_swingComponent.setBounds(getX(), getY(), getMaxWidth(), getMaxHeight());
		} else {
			_swingComponent.setBounds(getX(), getY(), getWidth(), getHeight());
		}
	}

	private void prepareToPaint()
	{
		_isReadyToPaint = true;
		layout(_swingComponent);
		ignoreAWTPainting(_swingComponent);
	}

	private void ignoreAWTPainting(Component c)
	{
		if (c == null) return;
		
		if (c instanceof JComponent) ((JComponent) c).setDoubleBuffered(false);
		c.setIgnoreRepaint(true);

		if (c instanceof Container) {
			for (Component child : ((Container) c).getComponents()) {
				ignoreAWTPainting(child);
			}
		}
	}

	/**
	 * Due to absolute positioning...
	 * 
	 * @param parent
	 */
	protected void layout(Component parent)
	{
		if (parent == null) return;
		
		parent.validate();

		if (parent instanceof Container) {
			for (Component child : ((Container) parent).getComponents()) {
				layout(child);
			}
		}
	}

	@Override
	protected final void paintWidget()
	{
		if (!_isReadyToPaint) {
			prepareToPaint();
		}

		Graphics2D g = SwingMiscManager.getIfUsingSwingGraphicsManager().getCurrentSurface();

		final Point location = _swingComponent.getLocation();
		int x = location.x;
		int y = location.y;
		
		g.translate(x, y);
		paintSwingWidget(g);
		g.translate(-x, -y);
	}
	
	/** Can be overridden to enable custom painting of widgets. */
	protected void paintSwingWidget(Graphics2D g)
	{
		_swingComponent.paint(g);
	}
	
	@Override
	public boolean isSupportedOnEcosystem(Ecosystem type)
	{
		return type == Ecosystem.Swing;
	}
}
